-- bitmap lib (luaDEV) by zerozelta

bmp = {}

function bmp.loadImage(path)

	if files.exists(path) == false then return end
	
	local imagen					-- Objeto de imagen que vamos a devolver
	
	local header = {} 				-- Inicializamos el header del bmp
	local infoHeader = {}			-- Inicializamos info header del bitmap
	local coreHeader = {}
	
	local imageData = {}			-- Galeria de pixeles para el bitmap
	
	fd = io.open(path,"rb")			-- Establecemos un descriptor de fisheros
	if fd == nil then fd:close(); return end 	-- Error al abrir este archivo
	
	header = bmp.getHeader(fd)					-- Obtenemos header 
	infoHeader = bmp.getInfoHeader(fd)			-- Obtenemos header info
	coreHeader = bmp.getCoreHeader(fd)
	
	system.mensaje("bmp data","","bpp: "..infoHeader.hRes.." | "..infoHeader.width.."x"..infoHeader.height.."   Compress: "..infoHeader.compression)
	
	if infoHeader.height > 512 or infoHeader.width > 512 then 
		fd:close(); return 					-- La imagen es muy grande
	end 
	
	if infoHeader.compression ~= 0 then fd:close(); return end 								-- Solo soportamos archivos sin compresion
	
	imagen = bmp.getImageFrom(fd,infoHeader)												-- Obtenemos el objeto imagen del bmp cargado
	
	if imagen == nil then fd:close(); return end
	
	fd:close();
	collectgarbage()
	
	return imagen;

end

function bmp.getHeader(fd)	-- Leemos el header del descriptor pasado como argumento, devolvemos una tabla con el header
		fd:seek("set",0)
		
		local header = {
		
		identity = fd:read(2),							-- Tabla de identidad tamao (String)
		size = string.byte(fd:read(4)),					-- 4 bytes | the size of the BMP file in bytes
		reserved = string.byte(fd:read(4)),				-- 4 bytes (no seran utilizados) (Esto incluye 2 secciones del header)
		offset = string.byte(fd:read(4)),				-- 4 bytes 
		
	}
	
	return header
end

function bmp.getInfoHeader(fd)
	fd:seek("set",16)
	
	local header = {			-- Inicializamos info header del bitmap
		size = string.byte(fd:read(4)),
		width =  bmp.readNumber(fd,4),
		height = bmp.readNumber(fd,4),
		colorPlanes = string.byte(fd:read(2)),
		colorDepth = string.byte(fd:read(2)),
		compression = string.byte(fd:read(4)),
		imageSize = string.byte(fd:read(4)),
		hRes = string.byte(fd:read(4)),
		vRes = string.byte(fd:read(4)),
		nColors = string.byte(fd:read(4)),
		nIColors = string.byte(fd:read(4))
		
	}
	
	return header
end

function bmp.readNumber(fd,n)
	result = 0
	
	for i = 1,n do
		local value = string.byte(fd:read(1))
		if value < 0 then
			value = 0
		end
		
		result = result + value
	end
	
	os.message(result)
	return result
end

function bmp.getCoreHeader(fd)
	fd:seek("set",14)
	
	local header = {
		size = string.byte(fd:read(4)),
		width = string.byte(fd:read(2)),
		height = string.byte(fd:read(2)),
		colorPlanes = string.byte(fd:read(2)),
		colorDeph = string.byte(fd:read(2))
	}
	
	return header
end

function bmp.getImageFrom(fd,infoHeader)
	--- 24 bits (3 bytes por pixel) Este formato no soporta alfa  :/  ... peor es nada XD
	--- 32 bits (4 bytes por pixel) Este formato si soporta alfa
	
	local img = image.create(infoHeader.width,infoHeader.height,0x0)	-- Creamos nuestra nueva imagen
	local x,y = 1,1
	local paso = 1
	
	local totalSize = fd:seek("end")									-- Entero total en tamao del archivo
	local totalRead = 54												-- Entero total en tamao leido
	local store = totalSize 
	
	if infoHeader.height < 0 then
		y = 1
		paso = 1
	else
		y = infoHeader.height
		paso = -1
	end	

	fd:seek("set",54)
	 
	while totalRead < totalSize do
		local RGB = fd:read(3)
		
		local B = string.byte(string.sub(RGB,1))
		local G = string.byte(string.sub(RGB,2))
		local R = string.byte(string.sub(RGB,3))
		local A = 255 
		
		img:pixel((x - 1),(y - 1),color.new(R,G,B,A));
		
		totalRead = totalRead + 3;
		
		x = x + 1;
		
		if x > infoHeader.width then
			x = 1;
			y = y + paso;
		end

	end
	
	return img
end